package dev.fyloz.colorrecipesexplorer.repository

import dev.fyloz.colorrecipesexplorer.model.Material
import dev.fyloz.colorrecipesexplorer.model.MaterialType
import org.springframework.data.jpa.repository.Modifying
import org.springframework.data.jpa.repository.Query
import org.springframework.stereotype.Repository

@Repository
interface MaterialRepository : NamedJpaRepository<Material> {
    /** Checks if one or more materials have the given [materialType]. */
    fun existsByMaterialType(materialType: MaterialType): Boolean

    /** Updates the [inventoryQuantity] of the [Material] with the given [id]. */
    @Modifying
    @Query("UPDATE Material m SET m.inventoryQuantity = :inventoryQuantity WHERE m.id = :id")
    fun updateInventoryQuantityById(id: Long, inventoryQuantity: Float)

    @Query(
      """
        select case when(count(mm.id) + count(mt.id) > 0) then false else true end
        from Material m
        left join MixMaterial mm on m.id = mm.material.id
        left join MixType mt on m.id = mt.material.id
        where m.id = :id
    """
    )
    fun canBeDeleted(id: Long): Boolean
}
